@@ -1,46 +1,17 @@
 import functools
 import warnings
 
-from astropy.coordinates import SkyCoord
 import astropy.units as u
 import numpy as np
 import sunpy.map
+from astropy.coordinates import SkyCoord
 
 import pfsspy.coords
-from pfsspy.grid import Grid
 
 # Default colourmap for magnetic field maps
 _MAG_CMAP = 'RdBu'
 
 
-def load_output(file):
-    """
-    Load a saved output file.
-
-    Loads a file saved using :meth:`Output.save`.
-
-    Parameters
-    ----------
-    file : str, file, :class:`~pathlib.Path`
-        File to load.
-
-    Returns
-    -------
-    :class:`Output`
-    """
-    with np.load(file) as f:
-        alr = f['alr']
-        als = f['als']
-        alp = f['alp']
-        rss = f['rss']
-    ns = als.shape[1]
-    nphi = alp.shape[0]
-    nr = alr.shape[2]
-    rss = rss[0]
-    grid = Grid(ns, nphi, nr, rss)
-    return Output(alr, als, alp, grid)
-
-
 class Output:
     '''
     Output of PFSS modelling.
@@ -74,25 +45,6 @@ class Output:
         self._common_b_cache = None
         self._rgi = None
 
-    def save(self, file):
-        """
-        Save the output to file.
-
-        This saves the required information to reconstruct an Output object
-        in a compressed binary numpy file (see :func:`numpy.savez_compressed`
-        for more information). The file extension is ``.npz``, and is
-        automatically added if not present.
-
-        Parameters
-        ----------
-        file : str, file, :class:`~pathlib.Path`
-            File to save to. If ``.npz`` extension isn't present it is added
-            when saving the file.
-        """
-        np.savez_compressed(
-            file, alr=self._alr, als=self._als, alp=self._alp,
-            rss=np.array([self.grid.rss]))
-
     def _wcs_header(self):
         """
         Construct a world coordinate system describing the pfsspy solution.
@@ -120,25 +72,30 @@ class Output:
 
     @property
     def dtime(self):
+        """
+        Date and time of the input map.
+        """
         return self.input_map.date
 
     @property
     def bunit(self):
         """
         `~astropy.units.Unit` of the input map data.
+
+        If no metadata is available, returns dimensionless units.
         """
         # Note that this can be removed once pfsspy depends on sunpy>=2.1, see
         # https://github.com/sunpy/sunpy/pull/4451
         unit_str = self.input_map.meta.get('bunit', None)
         if unit_str is None:
-            return
+            return u.dimensionless_unscaled
 
         unit = u.Unit(unit_str, format='fits', parse_strict='silent')
         if isinstance(unit, u.UnrecognizedUnit):
             warnings.warn(f'Could not parse unit string "{unit_str}" as a valid FITS unit.\n'
                           'See https://fits.gsfc.nasa.gov/fits_standard.html '
                           'for the FITS unit standards.')
-            unit = None
+            unit = u.dimensionless_unscaled
         return unit
 
     @property
@@ -151,9 +108,11 @@ class Output:
         :class:`sunpy.map.GenericMap`
         """
         # Get radial component at the top
-        br = self.bc[0][:, :, -1]
+        br = self.bc[0][:, :, -1].to_value(self.bunit)
         # Remove extra ghost cells off the edge of the grid
         m = sunpy.map.Map((br.T, self._wcs_header()))
+        if 'bunit' in self.input_map.meta:
+            m.meta['bunit'] = self.input_map.meta['bunit']
         vlim = np.max(np.abs(br))
         m.plot_settings['cmap'] = _MAG_CMAP
         m.plot_settings['vmin'] = -vlim
@@ -195,7 +154,7 @@ class Output:
         # Because we need the cartesian grid to stretch just beyond r=rss,
         # add an extra dummy layer of magnetic field pointing radially outwards
         rho = np.append(rho, rho[-1] + 0.01)
-        extras = np.ones(br.shape[0:2] + (1, ))
+        extras = np.ones(br.shape[0:2] + (1, )) * self.bunit
         br = np.concatenate((br, extras), axis=2).astype(f32)
         bth = np.concatenate((bth, 0 * extras), axis=2).astype(f32)
         bph = np.concatenate((bph, 0 * extras), axis=2).astype(f32)
@@ -291,9 +250,19 @@ class Output:
 
         Returns
         -------
-        br : numpy.ndarray
-        btheta : numpy.ndarray
-        bphi : numpy.ndarray
+        br : astropy.units.Quantity
+            A ``nphi, ns, nrho + 1`` shaped Quantity.
+        btheta : astropy.units.Quantity
+            A ``nphi, ns + 1, nrho`` shaped Quantity.
+        bphi : astropy.units.Quantity
+            A ``nphi + 1, ns, nrho`` shaped Quantity.
+
+        Notes
+        -----
+        The three components are not co-located at the same locations.
+        Component ``n`` is located on cell faces of constant ``n``, e.g.
+        the ``r`` component is located on the cell faces at constant ``r``
+        values.
         """
         br, bs, bp, Sbr, Sbs, Sbp = self._common_b()
         # Remove area factors:
@@ -307,7 +276,9 @@ class Output:
             bp[i, :, :] = bp[i, :, :] / Sbp
 
         # Slice to remove ghost cells
-        return br[1:-1, 1:-1, :], -bs[1:-1, :, 1:-1], bp[:, 1:-1, 1:-1]
+        return (br[1:-1, 1:-1, :] * self.bunit,
+                -bs[1:-1, :, 1:-1] * self.bunit,
+                bp[:, 1:-1, 1:-1] * self.bunit)
 
     @property
     @functools.lru_cache(maxsize=1)
@@ -317,10 +288,11 @@ class Output:
 
         Returns
         -------
-        numpy.ndarray
-            A ``(nphi + 1, ns + 1, nrho + 1, 3)`` shaped array.
+        astropy.units.Quantity
+            A ``(nphi + 1, ns + 1, nrho + 1, 3)`` shaped Quantity.
             The last index gives the corodinate axis, 0 for Bphi, 1 for Bs, 2
-            for Brho.
+            for Brho. Because the phi dimension is periodic,
+            ``bg[0, :, :] == bg[-1, :, :]``.
         """
         br, bs, bp, Sbr, Sbs, Sbp = self._common_b()
         # Weighted average to grid points:
@@ -336,7 +308,7 @@ class Output:
         bsg *= -1
         out = np.stack((bpg, bsg, brg), axis=-1)
         out.flags.writeable = False
-        return out
+        return out * self.bunit
 
     @property
     @functools.lru_cache(maxsize=1)
@@ -374,7 +346,7 @@ class Output:
         thc = np.zeros(ns + 2) - 1
         thc[1:-1] = np.arccos(sc)
         # Centre of cells in phi (including ghost cells)
-        pc = np.linspace(-0.5 * dp, 2 * np.pi + 0.5 * dp, nphi + 2)
+        np.linspace(-0.5 * dp, 2 * np.pi + 0.5 * dp, nphi + 2)
 
         # Required face normals:
         dnp = np.zeros((ns + 2, 2))
@@ -432,14 +404,14 @@ class Output:
         br[-1, :, :] = br[1, :, :]
         # js = jp = 0 at photosphere:
         for i in range(nphi + 1):
-            bp[i,:,0] = Sbp[:,0]/dnp[:,0]*(bp[i,:,1]*dnp[:,1]/Sbp[:,1] + br[i,:,0]*dnr[:]/Sbr[:,0] - br[i+1,:,0]*dnr[:]/Sbr[:,0])
+            bp[i, :, 0] = Sbp[:, 0] / dnp[:, 0] * (bp[i, :, 1] * dnp[:, 1] / Sbp[:, 1] + br[i, :, 0] * dnr[:] / Sbr[:, 0] - br[i + 1, :, 0] * dnr[:] / Sbr[:, 0])
         for i in range(nphi + 2):
-            bs[i,:,0] = Sbs[:,0]/dns[:,0]*(bs[i,:,1]*dns[:,1]/Sbs[:,1] + br[i,:-1,0]*dnr[:-1]/Sbr[:-1,0] - br[i,1:,0]*dnr[1:]/Sbr[1:,0])
+            bs[i, :, 0] = Sbs[:, 0] / dns[:, 0] * (bs[i, :, 1] * dns[:, 1] / Sbs[:, 1] + br[i, :-1, 0] * dnr[:-1] / Sbr[:-1, 0] - br[i, 1:, 0] * dnr[1:] / Sbr[1:, 0])
         # - polar boundaries as in dumfric:
         for i in range(nphi + 2):
-            i1 = (i + nphi//2) % nphi
-            br[i,-1,:] = br[i1,-2,:]
-            br[i,0,:] = br[i1,1,:]
+            i1 = (i + nphi // 2) % nphi
+            br[i, -1, :] = br[i1, -2, :]
+            br[i, 0, :] = br[i1, 1, :]
             bs[i, -1, :] = 0.5 * (bs[i, -2, :] - bs[i1, -2, :])
             bs[i, 0, :] = 0.5 * (bs[i, 1, :] - bs[i1, 1, :])
         for i in range(nphi + 1):
@@ -467,8 +439,8 @@ class Output:
 
         Returns
         -------
-        bvec : ndarray
-            (N, 3) shaped array of magnetic field vectors.
+        bvec : astropy.units.Quantity
+            (N, 3) shaped Quantity of magnetic field vectors.
 
         Notes
         -----
@@ -536,7 +508,5 @@ class Output:
                  np.cos(coords.lon).value,
                  np.zeros(len(coords))],
             ])
-            bvecs = np.array([np.dot(M_.T,v) for M_,v in zip(M.T,bvecs)])
-        if self.bunit is not None:
-            bvecs *= self.bunit
-        return bvecs
+            bvecs = np.array([np.dot(M_.T, v) for M_, v in zip(M.T, bvecs)])
+        return bvecs * self.bunit
